@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import multiprocessing
 from functools import partial
 from typing import Literal, Sequence, Union
 
@@ -42,46 +43,59 @@ class Zodipy:
     Attributes:
         model (str): Name of the interplanetary dust model to use in the simulations.
             Defaults to DIRBE.
-        ephemeris (str): Ephemeris used to compute the positions of the observer and the
-            Earth. Defaults to 'de432s' which requires downloading (and caching) a ~10MB
-            file. For more information on available ephemeridis, please visit
-            https://docs.astropy.org/en/stable/coordinates/solarsystem.html
         extrapolate (bool): If True all spectral quantities in the selected model are
             linearly extrapolated to the requested frequency/wavelength. If False, an
             Exception will be raised on requested frequencies/wavelengths outside of the
             valid model range. Default is False.
-        gauss_quad_order (int): Order of the Gaussian-Legendre quadrature used to evaluate
-            the brightness integral. Default is 50 points.
-        cutoff (u.Quantity[u.AU]): Radial distance from the Sun at which all line of sights
-            are truncated. Defaults to 5.2 AU which is the distance to Jupiter.
-        solar_cutoff (u.Quantity[u.deg]): Cutoff angle from the sun in degrees. The emission
+        parallel (bool): If True, input pointing sequences will be split among all
+            available cores on the machine, and the emission will be computed in parallel.
+            This is useful for large simulations. NOTE: This will require the user to guard
+            the executed code in a `if __name__ == "__main__"` block. Default is False.
+        n_proc (int): Number of cores to use when parallel computation. Defaults is None,
+            which will use all available cores.
+        solar_cut (u.Quantity[u.deg]): Cutoff angle from the sun in degrees. The emission
             for all the pointing with angular distance between the sun smaller than
             `solar_cutoff` are set to `np.nan`. This is due to the model singularity of the
             diffuse cloud component at the heliocentric origin. Such a cutoff may be
             useful when using simulated pointing, but actual scanning strategies are
             unlikely to look directly at the sun. This feature is turned of by setting this
             argument to `None`. Defaults to 5 degrees.
+        solar_cut_fill_value (float): Fill value for the masked solar cut pointing.
+            Defaults to `np.nan`.
+        gauss_quad_order (int): Order of the Gaussian-Legendre quadrature used to evaluate
+            the brightness integral. Default is 50 points.
+        los_dist_cut (u.Quantity[u.AU]): Radial distance from the Sun at which all line of
+            sights are truncated. Defaults to 5.2 AU which is the distance to Jupiter.
+        ephemeris (str): Ephemeris used to compute the positions of the observer and the
+            Earth. Defaults to 'de432s' which requires downloading (and caching) a ~10MB
+            file. For more information on available ephemeridis, please visit
+            https://docs.astropy.org/en/stable/coordinates/solarsystem.html
 
     """
 
     def __init__(
         self,
         model: str = "dirbe",
-        ephemeris: str = "de432s",
         extrapolate: bool = False,
+        parallel: bool = False,
+        n_proc: int | None = None,
+        solar_cut: u.Quantity[u.deg] | None = None,
+        solar_cut_fill_value: float = np.nan,
         gauss_quad_order: int = 100,
-        cutoff: u.Quantity[u.AU] = DISTANCE_TO_JUPITER,
-        solar_cutoff: u.Quantity[u.deg] | None = None,
+        los_dist_cut: u.Quantity[u.AU] = DISTANCE_TO_JUPITER,
+        ephemeris: str = "de432s",
     ) -> None:
 
         self.model = model_registry.get_model(model)
-        self.ephemeris = ephemeris
         self.extrapolate = extrapolate
-        self.cutoff = cutoff
-        self.solar_cutoff = (
-            solar_cutoff.to(u.rad) if solar_cutoff is not None else solar_cutoff
-        )
-        self.integration_scheme = quadpy.c1.gauss_legendre(gauss_quad_order)
+        self.parallel = parallel
+        self.n_proc = n_proc
+        self.solar_cut = solar_cut.to(u.rad) if solar_cut is not None else solar_cut
+        self.solar_cut_fill_value = solar_cut_fill_value
+        self.gauss_quad_order = gauss_quad_order
+        self.integration_scheme = quadpy.c1.gauss_legendre(self.gauss_quad_order)
+        self.los_dist_cut = los_dist_cut
+        self.ephemeris = ephemeris
 
     @property
     def supported_observers(self) -> list[str]:
@@ -368,26 +382,19 @@ class Zodipy:
             obs=obs, obs_time=obs_time, obs_pos=obs_pos
         )
 
-        if self.solar_cutoff is not None:
-            # The observer position is aquired in geocentric coordinates before being
-            # rotated to ecliptic coordinates which means that we can find the
-            # heliocentric origin by simply taking the negative of the observer position.
-            # We set all unit_vectors with a angular distance from the suns smaller than
-            # the cutoff to np.nan which will propagate to np.nan/hp.UNSEEN in the emission.
-            ang_dist = hp.rotator.angdist(-observer_position, unit_vectors)
-            unit_vectors[:, ang_dist < self.solar_cutoff.value] = np.nan
-
         if self.model.solar_irradiance_model is not None:
             solar_irradiance = (
                 self.model.solar_irradiance_model.interpolate_solar_irradiance(
-                    freq=freq, albedos=self.model.albedos, extrapolate=self.extrapolate
+                    freq=freq,
+                    albedos=self.model.albedos,
+                    extrapolate=self.extrapolate,
                 )
             )
         else:
             solar_irradiance = 0
 
         start, stop = get_line_of_sight_endpoints(
-            cutoff=self.cutoff.value,
+            cutoff=self.los_dist_cut.value,
             obs_pos=observer_position,
             unit_vectors=unit_vectors,
         )
@@ -397,57 +404,108 @@ class Zodipy:
         )
 
         # Preparing quantities for broadcasting
-        stop_expanded = np.expand_dims(stop, axis=-1)
-        observer_position = np.expand_dims(observer_position, axis=-1)
-        earth_position = np.expand_dims(earth_position, axis=-1)
-        unit_vectors = np.expand_dims(unit_vectors, axis=-1)
+        observer_position_expanded = np.expand_dims(observer_position, axis=-1)
+        earth_position_expanded = np.expand_dims(earth_position, axis=-1)
 
-        # For each component, compute the integrated line of sight emission
-        for idx, (label, comp) in enumerate(self.model.comps.items()):
-            source_parameters = self.model.interpolate_source_parameters(label, freq)
-            emissivity, albedo, phase_coefficients = source_parameters
+        emissivities, albedos, phase_coefficients = self.model.interp_source_params(
+            freq
+        )
+        # Distribute pointing to available CPUs and compute the emission in parallel.
+        if self.parallel:
+            n_proc = multiprocessing.cpu_count() if self.n_proc is None else self.n_proc
 
-            # Here we create a partial function that will be passed to the
-            # integration scheme. The arrays are reshaped to (d, n, p) where d
-            # is the geometrical dimensionality, n is the number of different
-            # pointings, and p is the number of integration points of the
-            # quadrature.
-            emission_comp_integrand = partial(
-                _compute_comp_emission_at_step,
+            unit_vector_chunks = np.array_split(unit_vectors, n_proc, axis=-1)
+            stop_chunks = np.array_split(stop, n_proc, axis=-1)
+
+            with multiprocessing.Pool(processes=n_proc) as pool:
+                # Here we create a partial function that will be passed to the
+                # integration scheme. The arrays are reshaped to (d, n, p) where d
+                # is the geometrical dimensionality, n is the number of different
+                # pointings, and p is the number of integration points of the
+                # quadrature.
+                partial_integrand_chunks = [
+                    partial(
+                        _compute_emission_at_step,
+                        start=start,
+                        stop=np.expand_dims(stop, axis=-1),
+                        n_quad_points=self.gauss_quad_order,
+                        X_obs=observer_position_expanded,
+                        X_earth=earth_position_expanded,
+                        u_los=np.expand_dims(unit_vectors, axis=-1),
+                        comps=list(self.model.comps.values()),
+                        freq=freq.value,
+                        T_0=self.model.T_0,
+                        delta=self.model.delta,
+                        emissivities=emissivities,
+                        albedos=albedos,
+                        phase_coefficients=phase_coefficients,
+                        solar_irradiance=solar_irradiance,
+                    )
+                    for unit_vectors, stop in zip(unit_vector_chunks, stop_chunks)
+                ]
+
+                proc_chunks = [
+                    pool.apply_async(
+                        self.integration_scheme.integrate,
+                        args=(partial_integrand, [-1, 1]),
+                    )
+                    for partial_integrand in partial_integrand_chunks
+                ]
+                integrated_comp_emission = np.concatenate(
+                    [result.get() for result in proc_chunks], axis=1
+                )
+
+        # Compute the emission only in the main process.
+        else:
+            unit_vectors_expanded = np.expand_dims(unit_vectors, axis=-1)
+            stop_expanded = np.expand_dims(stop, axis=-1)
+
+            partial_emission_integrand = partial(
+                _compute_emission_at_step,
                 start=start,
                 stop=stop_expanded,
-                X_obs=observer_position,
-                X_earth=earth_position,
-                u_los=unit_vectors,
-                comp=comp,
+                n_quad_points=self.gauss_quad_order,
+                X_obs=observer_position_expanded,
+                X_earth=earth_position_expanded,
+                u_los=unit_vectors_expanded,
+                comps=list(self.model.comps.values()),
                 freq=freq.value,
                 T_0=self.model.T_0,
                 delta=self.model.delta,
-                emissivity=emissivity,
-                albedo=albedo,
+                emissivities=emissivities,
+                albedos=albedos,
                 phase_coefficients=phase_coefficients,
                 solar_irradiance=solar_irradiance,
             )
 
             integrated_comp_emission = self.integration_scheme.integrate(
-                emission_comp_integrand, [-1, 1]
+                partial_emission_integrand, [-1, 1]
             )
-            # We convert the integral from [-1, 1] back to [start, stop].
-            integrated_comp_emission *= 0.5 * (stop - start)
 
-            if binned:
-                emission[idx, pixels] = integrated_comp_emission
-            else:
-                emission[idx] = integrated_comp_emission[indicies]
+        # We convert the integral from [-1, 1] back to [start, stop].
+        integrated_comp_emission *= 0.5 * (stop - start)
 
         if binned:
-            # We multiply the binned map by the number of hits.
-            emission[:, pixels] *= indicies
+            emission[:, pixels] = integrated_comp_emission
+        else:
+            emission = integrated_comp_emission[:, indicies]
+
+        if self.solar_cut is not None:
+            # The observer position is aquired in geocentric coordinates before being
+            # rotated to ecliptic coordinates. This means that we can find the
+            # heliocentric origin in this coordinate system by simply taking the negative
+            # of the observer position. The emission corresponding to unit_vectors with a
+            # angular distance (between the pointing and the sun) smaller than the
+            # specified `solar_cutoff` value is masked with the `solar_cutoff_fill_value`.
+            ang_dist = hp.rotator.angdist(-observer_position, unit_vectors)
+            solar_mask = ang_dist < self.solar_cut.value
+            if binned and pixels is not None:
+                emission[:, pixels[solar_mask]] = self.solar_cut_fill_value
+            else:
+                emission[:, solar_mask[indicies]] = self.solar_cut_fill_value
 
         return emission
 
-    # def _remove_masked_pointing
-
     def __repr__(self) -> str:
         return (
             f"{type(self).__name__}(model={self.model.name!r}, "
@@ -459,24 +517,25 @@ class Zodipy:
         return repr(self.model)
 
 
-def _compute_comp_emission_at_step(
+def _compute_emission_at_step(
     r: float | NDArray[np.floating],
     *,
     start: float,
     stop: float | NDArray[np.floating],
+    n_quad_points: int,
     X_obs: NDArray[np.floating],
     X_earth: NDArray[np.floating],
     u_los: NDArray[np.floating],
-    comp: Component,
+    comps: list[Component],
     freq: float,
     T_0: float,
     delta: float,
-    emissivity: float,
-    albedo: float,
+    emissivities: list[float],
+    albedos: list[float],
     phase_coefficients: tuple[float, float, float],
     solar_irradiance: float,
 ) -> NDArray[np.floating]:
-    """Returns the zodiacal emission at a step along a line of sight."""
+    """Returns the zodiacal emission at a step along all lines of sight."""
 
     # Convert the line of sight range from [-1, 1] to the true ecliptic positions
     R_los = ((stop - start) / 2) * r + (stop + start) / 2
@@ -485,17 +544,22 @@ def _compute_comp_emission_at_step(
     X_helio = X_los + X_obs
     R_helio = np.sqrt(X_helio[0] ** 2 + X_helio[1] ** 2 + X_helio[2] ** 2)
 
-    density = comp.compute_density(X_helio, X_earth=X_earth)
     temperature = get_dust_grain_temperature(R_helio, T_0, delta)
     blackbody_emission = get_blackbody_emission(freq, temperature)
 
-    emission = (1 - albedo) * (emissivity * blackbody_emission)
+    emission = np.zeros((len(comps), np.shape(X_helio)[1], n_quad_points))
+    density = np.zeros((len(comps), np.shape(X_helio)[1], n_quad_points))
 
-    if albedo > 0:
+    for idx, (comp, albedo, emissivity) in enumerate(zip(comps, albedos, emissivities)):
+        density[idx] = comp.compute_density(X_helio, X_earth=X_earth)
+        emission[idx] = (1 - albedo) * (emissivity * blackbody_emission)
+
+    if any(albedo != 0 for albedo in albedos):
         solar_flux = solar_irradiance / R_helio**2
         scattering_angle = get_scattering_angle(R_los, R_helio, X_los, X_helio)
         phase_function = get_phase_function(scattering_angle, phase_coefficients)
 
-        emission += albedo * solar_flux * phase_function
+        for idx, albedo in enumerate(albedos):
+            emission[idx] += albedo * solar_flux * phase_function
 
     return emission * density
